"""
addons/eventscripts/smauth_service/smauth_service.py
Version 1.00.002

Licensed under:
http://creativecommons.org/licenses/by-nc-nd/3.0/

If wish to alter the file/contents or reuse some parts please contact me.

(C) [#OMEGA] - K2 2010
"""

# ##### #
# Psyco #
# ##### #

import psyco
psyco.full()

# #################### #
# Eventscripts Imports #
# #################### #

# Default
import es
import services
import services.auth

# Custom
import smauth

# ############## #
# Python Imports #
# ############## #

# ########### #
# Script Info #
# ########### #

info = es.AddonInfo()
info.name = "Sourcemod Authprovider Service"
info.version = "1.00.002"
info.author = "[#OMEGA] - K2"
info.url = "http://www.german-slaughterhouse.de"
info.description = "Provides auth access to sm files using the authorisation interface of es."
info.tags = "sm sourcemod auth provider"

# ############# #
# Load & Unload #
# ############# #

def load():
	services.register("auth", SMAuthService())
	# Public Vars
	es.set('es_smauth_service_ver', info.version)
	es.flags('add','notify','es_smauth_service_ver')
	es.set('es_smauth_service_ver', info.version)
	
def unload():
	services.unregister("auth")
	# Public Vars
	es.flags('remove','notify','es_smauth_service_ver')
	es.set('es_smauth_service_ver','Unloaded')
	
# ####### #
# Classes #
# ####### #

class InputError(Exception):
	pass
	
class PlayerAccess(smauth.PlayerAccess):
	def __init__(self,steamid):
		if steamid.startswith('STEAM_'):
			steamid = steamid[8:]
		self.authinfo = steamid
		self.authtype = 'steam'
		self.user = smauth.Admin.GetUser(self.authinfo,'steam')

class SMAuthService(services.auth.AuthorizationService):
	def __init__(self):
		#self.interfacename = None
		self.version = info.version
		self.name = "smauth"
		self.capabilities = {}
	def registerCapability(self, auth_capability, auth_recommendedlevel=services.auth.AuthorizationService.ADMIN):
		"""
		Registers a new capability.
		
		Raises InputError incase the given capability exists.
		"""
		if self.capabilities.has_key(auth_capability):
			es.dbgmsg(0, 'smauth/service.py: Capability "%s" already exists! Overriding.' % auth_capability)
		
		self.capabilities[auth_capability] = auth_recommendedlevel
		
	def isUseridAuthorized(self, auth_userid, auth_capability):
		"""	
		See isIdAuthoriszed for more info.
		"""
		return self.isIdAuthorized(self.getOfflineIdentifier(auth_userid),auth_capability)

	def getOfflineIdentifier(self, auth_userid):
		"""
		Returns the steamid of the specified userid.
		"""
		return es.getplayersteamid(auth_userid)

	def isIdAuthorized(self, auth_identifier, auth_capability):
		"""
		This function maps the ES access levels for the given capability to sourcemod access levels. It will use the steamid as identifier - if you wish to authentificate users per ip or name you'll need to use the smauth libary directly.
		As such, the following will be applied:
		
		UNRESTRICTED = Everyone
		IDENTIFIED = Any user present in the sourcemod authentification files
		POWERUSER = Any user with a flag (reserved slot access)
		ADMIN = Any user with b flag (basic admin access)
		ROOT = Any user with z flag (root access)
		
		Raises InputError incase the specified capability does not exist.
		"""
		if not self.capabilities.has_key(auth_capability):
			raise InputError, 'Can not check permissions for capability "%s". It does not exist!' % auth_capability
		
		player = PlayerAccess(auth_identifier)
		
		auth_level = self.capabilities[auth_capability]
		
		if auth_level == self.UNRESTRICTED:
			return True
		elif auth_level == self.IDENTIFIED:
			return player.IsUser()
		elif auth_level == self.POWERUSER:
			return player.HasFlag('a')
		elif auth_level == self.ADMIN:
			return player.IsAdmin()
		elif auth_level == self.ROOT:
			return player.HasFlag('z')
		else:
			return False